/*
 * evm.c
 *
 * Board functions for TI814x EVM
 *
 * Copyright (C) 2011, Texas Instruments, Incorporated - http://www.ti.com/
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#include <common.h>
#include <cpsw.h>
#include <errno.h>
#include <spl.h>
#include <asm/arch/cpu.h>
#include <asm/arch/cpu_ti814x.h>

#include <asm/arch/hardware.h>
#include <asm/arch/omap.h>
#include <asm/arch/ddr_defs.h>
#include <asm/arch/ddr_defs_ti814x.h>

#include <asm/arch/clock.h>
#include <asm/arch/gpio.h>
#include <asm/arch/mmc_host_def.h>
#include <asm/arch/sys_proto.h>
#include <asm/io.h>
#include <asm/emif.h>
#include <asm/gpio.h>
#include "evm.h"
#include <environment.h>

DECLARE_GLOBAL_DATA_PTR;

static struct ctrl_dev *cdev = (struct ctrl_dev *)CTRL_DEVICE_BASE;

static void gpio_clkctrl_enable(void)
{
	__raw_writel(0x102, CM_ALWON_GPIO_0_CLKCTRL);
	while((__raw_readl(CM_ALWON_GPIO_0_CLKCTRL) & 0x3) != 0x2);

	__raw_writel(0x102, CM_ALWON_GPIO_1_CLKCTRL);
	while((__raw_readl(CM_ALWON_GPIO_1_CLKCTRL) & 0x3) != 0x2);
}
void led_init(void)
{
	u8 i;
	gpio_clkctrl_enable();
	for(i=24;i<=27;i++)
	{
		
		gpio_direction_output(i,1);//AP_LED, LED_RED, LED_YG,REC_LED off
		gpio_set_value(i,1);
	}
	gpio_direction_output(28,0);
	gpio_set_value(28,0);

}

/* UART Defines */
#ifdef CONFIG_SPL_BUILD

static void cmd_macro_config(u32 ddr_phy, u32 inv_clk_out,
			 u32 ctrl_slave_ratio_cs0, u32 cmd_dll_lock_diff)
{
	u32 ddr_phy_base = (DDR_PHY0 == ddr_phy) ?
			 DDR0_PHY_BASE_ADDR : DDR1_PHY_BASE_ADDR;

	writel(inv_clk_out,
		 ddr_phy_base + CMD1_REG_PHY_INVERT_CLKOUT_0);
	writel(inv_clk_out,
		 ddr_phy_base + CMD0_REG_PHY_INVERT_CLKOUT_0);
	writel(inv_clk_out,
		 ddr_phy_base + CMD2_REG_PHY_INVERT_CLKOUT_0);

	writel(((ctrl_slave_ratio_cs0 << 10) | ctrl_slave_ratio_cs0),
		ddr_phy_base + CMD0_REG_PHY_CTRL_SLAVE_RATIO_0);
	writel(((ctrl_slave_ratio_cs0 << 10) | ctrl_slave_ratio_cs0),
		ddr_phy_base + CMD1_REG_PHY_CTRL_SLAVE_RATIO_0);
	writel(((ctrl_slave_ratio_cs0 << 10) | ctrl_slave_ratio_cs0),
		 ddr_phy_base + CMD2_REG_PHY_CTRL_SLAVE_RATIO_0);

	writel(cmd_dll_lock_diff,
		 ddr_phy_base + CMD0_REG_PHY_DLL_LOCK_DIFF_0);
	writel(cmd_dll_lock_diff,
		 ddr_phy_base + CMD1_REG_PHY_DLL_LOCK_DIFF_0);
	writel(cmd_dll_lock_diff,
		 ddr_phy_base + CMD2_REG_PHY_DLL_LOCK_DIFF_0);
}

static void data_macro_config(u32 macro_num, u32 emif, u32 rd_dqs_cs0,
		u32 wr_dqs_cs0, u32 fifo_we_cs0, u32 wr_data_cs0)
{
	/* 0xA4 is size of each data macro mmr region.
	 * phy1 is at offset 0x400 from phy0
	 */
	u32 base = (macro_num * 0xA4) + (emif * 0x400);

	writel(((rd_dqs_cs0 << 10) | rd_dqs_cs0),
		(DATA0_REG_PHY0_RD_DQS_SLAVE_RATIO_0 + base));
	writel(((wr_dqs_cs0 << 10) | wr_dqs_cs0),
		(DATA0_REG_PHY0_WR_DQS_SLAVE_RATIO_0 + base));
	writel(((PHY_WRLVL_INIT_CS1_DEFINE << 10) |
		PHY_WRLVL_INIT_CS0_DEFINE),
		(DATA0_REG_PHY0_WRLVL_INIT_RATIO_0 + base));
	writel(((PHY_GATELVL_INIT_CS1_DEFINE << 10) |
		PHY_GATELVL_INIT_CS0_DEFINE),
		(DATA0_REG_PHY0_GATELVL_INIT_RATIO_0 + base));
	writel(((fifo_we_cs0 << 10) | fifo_we_cs0),
		(DATA0_REG_PHY0_FIFO_WE_SLAVE_RATIO_0 + base));
	writel(((wr_data_cs0 << 10) | wr_data_cs0),
		(DATA0_REG_PHY0_WR_DATA_SLAVE_RATIO_0 + base));
	writel(PHY_DLL_LOCK_DIFF_DEFINE,
		(DATA0_REG_PHY0_DLL_LOCK_DIFF_0 + base));
}


static void config_ti814x_ddr(void)
{
	int emif;


	/*Enable the Power Domain Transition of L3 Fast Domain Peripheral*/
	writel(0x2, CM_DEFAULT_FW_CLKCTRL);
	/*Enable the Power Domain Transition of L3 Fast Domain Peripheral*/
	writel(0x2, CM_DEFAULT_L3_FAST_CLKSTCTRL);
	writel(0x2, CM_DEFAULT_EMIF_0_CLKCTRL); /*Enable EMIF0 Clock*/
	writel(0x2, CM_DEFAULT_EMIF_1_CLKCTRL); /*Enable EMIF1 Clock*/
	writel(0x2, CM_DEFAULT_DMM_CLKCTRL);

	/*Poll for L3_FAST_GCLK  & DDR_GCLK  are active*/
	while ((readl(CM_DEFAULT_L3_FAST_CLKSTCTRL) & 0x300) != 0x300);
	/*Poll for Module is functional*/
	while ((readl(CM_DEFAULT_EMIF_0_CLKCTRL)) != 0x2);
	while ((readl(CM_DEFAULT_EMIF_1_CLKCTRL)) != 0x2);
	while ((readl(CM_DEFAULT_DMM_CLKCTRL)) != 0x2);


	cmd_macro_config(DDR_PHY0, DDR3_PHY_INVERT_CLKOUT_OFF,
			DDR3_PHY_CTRL_SLAVE_RATIO_CS0_DEFINE,
			PHY_CMD0_DLL_LOCK_DIFF_DEFINE);
	cmd_macro_config(DDR_PHY1, DDR3_PHY_INVERT_CLKOUT_OFF,
			DDR3_PHY_CTRL_SLAVE_RATIO_CS0_DEFINE,
			PHY_CMD0_DLL_LOCK_DIFF_DEFINE);

	 for (emif = 0; emif <= DDR_PHY1; emif++) 
	 {            
	 	data_macro_config(DATA_MACRO_0, emif,
			DDR3_PHY_RD_DQS_CS0_BYTE0,
			DDR3_PHY_WR_DQS_CS0_BYTE0,
			DDR3_PHY_RD_DQS_GATE_CS0_BYTE0,
			DDR3_PHY_WR_DATA_CS0_BYTE0);
		data_macro_config(DATA_MACRO_1, emif,
			DDR3_PHY_RD_DQS_CS0_BYTE1,
			DDR3_PHY_WR_DQS_CS0_BYTE1,
			DDR3_PHY_RD_DQS_GATE_CS0_BYTE1,
			DDR3_PHY_WR_DATA_CS0_BYTE1);
		data_macro_config(DATA_MACRO_2, emif,
			DDR3_PHY_RD_DQS_CS0_BYTE2,
			DDR3_PHY_WR_DQS_CS0_BYTE2,
			DDR3_PHY_RD_DQS_GATE_CS0_BYTE2,
			DDR3_PHY_WR_DATA_CS0_BYTE2);
		data_macro_config(DATA_MACRO_3, emif,
			DDR3_PHY_RD_DQS_CS0_BYTE3,
			DDR3_PHY_WR_DQS_CS0_BYTE3,
			DDR3_PHY_RD_DQS_GATE_CS0_BYTE3,
			DDR3_PHY_WR_DATA_CS0_BYTE3);
	}


	/* DDR IO CTRL config */
	writel(DDR0_IO_CTRL_DEFINE, DDR0_IO_CTRL);
	writel(DDR1_IO_CTRL_DEFINE, DDR1_IO_CTRL);

	writel(readl(VTP0_CTRL_REG) | 0x00000040 , VTP0_CTRL_REG);
	writel(readl(VTP1_CTRL_REG) | 0x00000040 , VTP1_CTRL_REG);

	// Write 0 to CLRZ bit
	writel(readl(VTP0_CTRL_REG) & 0xfffffffe , VTP0_CTRL_REG);
	writel(readl(VTP1_CTRL_REG) & 0xfffffffe , VTP1_CTRL_REG);

	// Write 1 to CLRZ bit
	writel(readl(VTP0_CTRL_REG) | 0x00000001 , VTP0_CTRL_REG);
	writel(readl(VTP1_CTRL_REG) | 0x00000001 , VTP1_CTRL_REG);

	// Read VTP control registers & check READY bits
	while ((readl(VTP0_CTRL_REG) & 0x00000020) != 0x20);
	while ((readl(VTP1_CTRL_REG) & 0x00000020) != 0x20);


	/*
	 * Program the PG2.1 DMM to Access EMIF0 and EMIF1
	 * 1G contiguous section with 128-byte interleaving
	 */

	writel(PG2_1_DMM_LISA_MAP__0, DMM_LISA_MAP__0);
	writel(PG2_1_DMM_LISA_MAP__1, DMM_LISA_MAP__1);
	writel(PG2_1_DMM_LISA_MAP__2, DMM_LISA_MAP__2);
	writel(PG2_1_DMM_LISA_MAP__3, DMM_LISA_MAP__3);


	while (readl(DMM_LISA_MAP__0) != PG2_1_DMM_LISA_MAP__0);
	while (readl(DMM_LISA_MAP__1) != PG2_1_DMM_LISA_MAP__1);
	while (readl(DMM_LISA_MAP__2) != PG2_1_DMM_LISA_MAP__2);
	while (readl(DMM_LISA_MAP__3) != PG2_1_DMM_LISA_MAP__3);


	writel(0x80000000, DMM_PAT_BASE_ADDR);

	/*Program EMIF0 CFG Registers*/
	writel(DDR3_EMIF_READ_LATENCY, EMIF4_0_DDR_PHY_CTRL_1);
	writel(DDR3_EMIF_READ_LATENCY, EMIF4_0_DDR_PHY_CTRL_1_SHADOW);
	writel(DDR3_EMIF_TIM1, EMIF4_0_SDRAM_TIM_1);
	writel(DDR3_EMIF_TIM1, EMIF4_0_SDRAM_TIM_1_SHADOW);
	writel(DDR3_EMIF_TIM2, EMIF4_0_SDRAM_TIM_2);
	writel(DDR3_EMIF_TIM2, EMIF4_0_SDRAM_TIM_2_SHADOW);
	writel(DDR3_EMIF_TIM3, EMIF4_0_SDRAM_TIM_3);
	writel(DDR3_EMIF_TIM3, EMIF4_0_SDRAM_TIM_3_SHADOW);
	writel(DDR3_EMIF_SDRAM_CONFIG, EMIF4_0_SDRAM_CONFIG);

	writel(DDR_EMIF_REF_CTRL | DDR_EMIF_REF_TRIGGER,
					 EMIF4_0_SDRAM_REF_CTRL);
	writel(DDR_EMIF_REF_CTRL, EMIF4_0_SDRAM_REF_CTRL_SHADOW);
	writel(DDR3_EMIF_SDRAM_ZQCR, EMIF4_0_SDRAM_ZQCR);
	writel(DDR_EMIF_REF_CTRL, EMIF4_0_SDRAM_REF_CTRL);
	writel(DDR_EMIF_REF_CTRL, EMIF4_0_SDRAM_REF_CTRL_SHADOW);

	writel(DDR3_EMIF_REF_CTRL, EMIF4_0_SDRAM_REF_CTRL);
	writel(DDR3_EMIF_REF_CTRL, EMIF4_0_SDRAM_REF_CTRL_SHADOW);

	/*Program EMIF1 CFG Registers*/
	writel(DDR3_EMIF_READ_LATENCY, EMIF4_1_DDR_PHY_CTRL_1);
	writel(DDR3_EMIF_READ_LATENCY, EMIF4_1_DDR_PHY_CTRL_1_SHADOW);
	writel(DDR3_EMIF_TIM1, EMIF4_1_SDRAM_TIM_1);
	writel(DDR3_EMIF_TIM1, EMIF4_1_SDRAM_TIM_1_SHADOW);
	writel(DDR3_EMIF_TIM2, EMIF4_1_SDRAM_TIM_2);
	writel(DDR3_EMIF_TIM2, EMIF4_1_SDRAM_TIM_2_SHADOW);
	writel(DDR3_EMIF_TIM3, EMIF4_1_SDRAM_TIM_3);
	writel(DDR3_EMIF_TIM3, EMIF4_1_SDRAM_TIM_3_SHADOW);
	writel(DDR3_EMIF_SDRAM_CONFIG, EMIF4_1_SDRAM_CONFIG);

	writel(DDR_EMIF_REF_CTRL | DDR_EMIF_REF_TRIGGER,
					 EMIF4_1_SDRAM_REF_CTRL);
	writel(DDR_EMIF_REF_CTRL, EMIF4_1_SDRAM_REF_CTRL_SHADOW);
	writel(DDR3_EMIF_SDRAM_ZQCR, EMIF4_1_SDRAM_ZQCR);
	writel(DDR_EMIF_REF_CTRL, EMIF4_1_SDRAM_REF_CTRL);
	writel(DDR_EMIF_REF_CTRL, EMIF4_1_SDRAM_REF_CTRL_SHADOW);

	writel(DDR3_EMIF_REF_CTRL, EMIF4_1_SDRAM_REF_CTRL);
	writel(DDR3_EMIF_REF_CTRL, EMIF4_1_SDRAM_REF_CTRL_SHADOW);


}


#define PADCTRL_BASE 0x48140000

#define PAD204_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B2c))
#define PAD205_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B30))
#define PAD206_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B34))
#define PAD207_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B38))
#define PAD208_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B3c))
#define PAD209_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B40))
#define PAD210_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B44))
#define PAD211_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B48))
#define PAD212_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B4c))
#define PAD213_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B50))
#define PAD214_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B54))
#define PAD215_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B58))
#define PAD216_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B5c))
#define PAD217_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B60))
#define PAD218_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B64))
#define PAD219_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B68))
#define PAD220_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B6c))
#define PAD221_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B70))
#define PAD222_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B74))
#define PAD223_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B78))
#define PAD224_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B7c))
#define PAD225_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B80))
#define PAD226_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B84))
#define PAD227_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B88))

#define PAD232_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0B9C))
#define PAD233_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BA0))
#define PAD234_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BA4))
#define PAD235_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BA8))
#define PAD236_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BAC))
#define PAD237_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BB0))
#define PAD238_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BB4))
#define PAD239_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BB8))
#define PAD240_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BBC))
#define PAD241_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BC0))
#define PAD242_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BC4))
#define PAD243_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BC8))
#define PAD244_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BCC))
#define PAD245_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BD0))
#define PAD246_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BD4))
#define PAD247_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BD8))
#define PAD248_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BDC))
#define PAD249_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BE0))
#define PAD250_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BE4))
#define PAD251_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BE8))
#define PAD252_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BEC))
#define PAD253_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BF0))
#define PAD254_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BF4))
#define PAD255_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BF8))
#define PAD256_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0BFC))
#define PAD257_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0C00))
#define PAD258_CNTRL  (*(volatile unsigned int *)(PADCTRL_BASE + 0x0C04))

#ifdef CONFIG_DRIVER_TI_CPSW
static void cpsw_pad_config()
{
	volatile u32 val = 0;

	/*configure pin mux for rmii_refclk,mdio_clk,mdio_d */
	val = PAD232_CNTRL;
	PAD232_CNTRL = (volatile unsigned int) (BIT(18) | BIT(0));
	val = PAD233_CNTRL;
	PAD233_CNTRL = (volatile unsigned int) (BIT(19) | BIT(17) | BIT(0));
	val = PAD234_CNTRL;
	PAD234_CNTRL = (volatile unsigned int) (BIT(19) | BIT(18) | BIT(17) |
			BIT(0));

	/*For PG1.0 we only support GMII Mode, setup gmii0/gmii1 pins here*/
	if (0) {
		/* setup gmii0 pins, pins235-258 in function mode 1 */
		val = PAD235_CNTRL;
		PAD235_CNTRL = (volatile unsigned int) (BIT(19) | BIT(18) |
				BIT(0));
		val = PAD236_CNTRL;
		PAD236_CNTRL = (volatile unsigned int) (BIT(19) | BIT(18) |
				BIT(0));
		val = PAD237_CNTRL;
		PAD237_CNTRL = (volatile unsigned int) (BIT(19) | BIT(18) |
				BIT(0));
		val = PAD238_CNTRL;
		PAD238_CNTRL = (volatile unsigned int) (BIT(19) | BIT(18) |
				BIT(0));
		val = PAD239_CNTRL;
		PAD239_CNTRL = (volatile unsigned int) (BIT(19) | BIT(18) |
				BIT(0));
		val = PAD240_CNTRL;
		PAD240_CNTRL = (volatile unsigned int) (BIT(18) | BIT(0));
		val = PAD241_CNTRL;
		PAD241_CNTRL = (volatile unsigned int) (BIT(18) | BIT(0));
		val = PAD242_CNTRL;
		PAD242_CNTRL = (volatile unsigned int) (BIT(18) | BIT(0));
		val = PAD243_CNTRL;
		PAD243_CNTRL = (volatile unsigned int) (BIT(18) | BIT(0));
		val = PAD244_CNTRL;
		PAD244_CNTRL = (volatile unsigned int) (BIT(18) | BIT(0));
		val = PAD245_CNTRL;
		PAD245_CNTRL = (volatile unsigned int) (BIT(18) | BIT(0));
		val = PAD246_CNTRL;
		PAD246_CNTRL = (volatile unsigned int) (BIT(18) | BIT(0));
		val = PAD247_CNTRL;
		PAD247_CNTRL = (volatile unsigned int) (BIT(18) | BIT(0));
		val = PAD248_CNTRL;
		PAD248_CNTRL = (volatile unsigned int) (BIT(18) | BIT(0));
		val = PAD249_CNTRL;
		PAD249_CNTRL = (volatile unsigned int) (BIT(0));
		val = PAD250_CNTRL;
		PAD250_CNTRL = (volatile unsigned int) (BIT(0));
		val = PAD251_CNTRL;
		PAD251_CNTRL = (volatile unsigned int) (BIT(0));
		val = PAD252_CNTRL;
		PAD252_CNTRL = (volatile unsigned int) (BIT(0));
		val = PAD253_CNTRL;
		PAD253_CNTRL = (volatile unsigned int) (BIT(0));
		val = PAD254_CNTRL;
		PAD254_CNTRL = (volatile unsigned int) (BIT(0));
		val = PAD255_CNTRL;
		PAD255_CNTRL = (volatile unsigned int) (BIT(0));
		val = PAD256_CNTRL;
		PAD256_CNTRL = (volatile unsigned int) (BIT(0));
		val = PAD257_CNTRL;
		PAD257_CNTRL = (volatile unsigned int) (BIT(0));
		val = PAD258_CNTRL;
		PAD258_CNTRL = (volatile unsigned int) (BIT(0));

		/* setup gmii1 pins, pins204-227 in function mode 2 */
		val = PAD204_CNTRL;
		PAD204_CNTRL = (volatile unsigned int) (BIT(19) | BIT(18) | BIT(1));
		val = PAD205_CNTRL;
		PAD205_CNTRL = (volatile unsigned int) (BIT(19) | BIT(18) | BIT(1));
		val = PAD206_CNTRL;
		PAD206_CNTRL = (volatile unsigned int) (BIT(19) | BIT(18) | BIT(1));
		val = PAD207_CNTRL;
		PAD207_CNTRL = (volatile unsigned int) (BIT(19) | BIT(18) | BIT(1));
		val = PAD208_CNTRL;
		PAD208_CNTRL = (volatile unsigned int) (BIT(19) | BIT(18) | BIT(1));
		val = PAD209_CNTRL;
		PAD209_CNTRL = (volatile unsigned int) (BIT(18) | BIT(1));
		val = PAD210_CNTRL;
		PAD210_CNTRL = (volatile unsigned int) (BIT(18) | BIT(1));
		val = PAD211_CNTRL;
		PAD211_CNTRL = (volatile unsigned int) (BIT(18) | BIT(1));
		val = PAD212_CNTRL;
		PAD212_CNTRL = (volatile unsigned int) (BIT(18) | BIT(1));
		val = PAD213_CNTRL;
		PAD213_CNTRL = (volatile unsigned int) (BIT(18) | BIT(1));
		val = PAD214_CNTRL;
		PAD214_CNTRL = (volatile unsigned int) (BIT(18) | BIT(1));
		val = PAD215_CNTRL;
		PAD215_CNTRL = (volatile unsigned int) (BIT(18) | BIT(1));
		val = PAD216_CNTRL;
		PAD216_CNTRL = (volatile unsigned int) (BIT(18) | BIT(1));
		val = PAD217_CNTRL;
		PAD217_CNTRL = (volatile unsigned int) (BIT(18) | BIT(1));
		val = PAD218_CNTRL;
		PAD218_CNTRL = (volatile unsigned int) (BIT(1));
		val = PAD219_CNTRL;
		PAD219_CNTRL = (volatile unsigned int) (BIT(1));
		val = PAD220_CNTRL;
		PAD220_CNTRL = (volatile unsigned int) (BIT(1));
		val = PAD221_CNTRL;
		PAD221_CNTRL = (volatile unsigned int) (BIT(1));
		val = PAD222_CNTRL;
		PAD222_CNTRL = (volatile unsigned int) (BIT(1));
		val = PAD223_CNTRL;
		PAD223_CNTRL = (volatile unsigned int) (BIT(1));
		val = PAD224_CNTRL;
		PAD224_CNTRL = (volatile unsigned int) (BIT(1));
		val = PAD225_CNTRL;
		PAD225_CNTRL = (volatile unsigned int) (BIT(1));
		val = PAD226_CNTRL;
		PAD226_CNTRL = (volatile unsigned int) (BIT(1));
		val = PAD227_CNTRL;
		PAD227_CNTRL = (volatile unsigned int) (BIT(1));

	} else {/*setup rgmii0/rgmii1 pins here*/
		/* In this case we enable rgmii_en bit in GMII_SEL register and
		 * still program the pins in gmii mode: gmii0 pins in mode 1*/
		val = PAD235_CNTRL; /*rgmii0_rxc*/
		PAD235_CNTRL = (volatile unsigned int) (BIT(18) | BIT(0));
		val = PAD236_CNTRL; /*rgmii0_rxctl*/
		PAD236_CNTRL = (volatile unsigned int) (BIT(18) | BIT(0));
		val = PAD237_CNTRL; /*rgmii0_rxd[2]*/
		PAD237_CNTRL = (volatile unsigned int) (BIT(18) | BIT(0));
		val = PAD238_CNTRL; /*rgmii0_txctl*/
		PAD238_CNTRL = (volatile unsigned int) BIT(0);
		val = PAD239_CNTRL; /*rgmii0_txc*/
		PAD239_CNTRL = (volatile unsigned int) BIT(0);
		val = PAD240_CNTRL; /*rgmii0_txd[0]*/
		PAD240_CNTRL = (volatile unsigned int) BIT(0);
		val = PAD241_CNTRL; /*rgmii0_rxd[0]*/
		PAD241_CNTRL = (volatile unsigned int) (BIT(18) | BIT(0));
		val = PAD242_CNTRL; /*rgmii0_rxd[1]*/
		PAD242_CNTRL = (volatile unsigned int) (BIT(18) | BIT(0));
		val = PAD243_CNTRL; /*rgmii1_rxctl*/
		PAD243_CNTRL = (volatile unsigned int) (BIT(18) | BIT(0));
		val = PAD244_CNTRL; /*rgmii0_rxd[3]*/
		PAD244_CNTRL = (volatile unsigned int) (BIT(18) | BIT(0));
		val = PAD245_CNTRL; /*rgmii0_txd[3]*/
		PAD245_CNTRL = (volatile unsigned int) BIT(0);
		val = PAD246_CNTRL; /*rgmii0_txd[2]*/
		PAD246_CNTRL = (volatile unsigned int) BIT(0);
		val = PAD247_CNTRL; /*rgmii0_txd[1]*/
		PAD247_CNTRL = (volatile unsigned int) BIT(0);
		val = PAD248_CNTRL; /*rgmii1_rxd[1]*/
		PAD248_CNTRL = (volatile unsigned int) (BIT(18) | BIT(0));
		val = PAD249_CNTRL; /*rgmii1_rxc*/
		PAD249_CNTRL = (volatile unsigned int) (BIT(18) | BIT(0));
		val = PAD250_CNTRL; /*rgmii1_rxd[3]*/
		PAD250_CNTRL = (volatile unsigned int) (BIT(18) | BIT(0));
		val = PAD251_CNTRL; /*rgmii1_txd[1]*/
		PAD251_CNTRL = (volatile unsigned int) (BIT(0));
#if 1 // hbchen		
		val = PAD252_CNTRL; /*rgmii1_txctl*/
		PAD252_CNTRL = (volatile unsigned int) (BIT(18) | BIT(1));
		val = PAD253_CNTRL; /*rgmii1_txd[0]*/
		PAD253_CNTRL = (volatile unsigned int) (BIT(18) | BIT(1));
		val = PAD254_CNTRL; /*rgmii1_txd[2]*/
		PAD254_CNTRL = (volatile unsigned int) (BIT(18) | BIT(1));
		val = PAD255_CNTRL; /*rgmii1_txc*/
		PAD255_CNTRL = (volatile unsigned int) (BIT(18) | BIT(1));
		val = PAD256_CNTRL; /*rgmii1_rxd[0]*/
		PAD256_CNTRL = (volatile unsigned int) (BIT(18) | BIT(1));
		val = PAD257_CNTRL; /*rgmii1_txd[3]*/
		PAD257_CNTRL = (volatile unsigned int) (BIT(18) | BIT(1));
		val = PAD258_CNTRL; /*rgmii1_rxd[2]*/
		PAD258_CNTRL = (volatile unsigned int) (BIT(18) | BIT(1));
#else
		PAD252_CNTRL = (volatile unsigned int) (BIT(0));
		val = PAD253_CNTRL; /*rgmii1_txd[0]*/
		PAD253_CNTRL = (volatile unsigned int) (BIT(0));
		val = PAD254_CNTRL; /*rgmii1_txd[2]*/
		PAD254_CNTRL = (volatile unsigned int) (BIT(0));
		val = PAD255_CNTRL; /*rgmii1_txc*/
		PAD255_CNTRL = (volatile unsigned int) (BIT(0));
		val = PAD256_CNTRL; /*rgmii1_rxd[0]*/
		PAD256_CNTRL = (volatile unsigned int) (BIT(18) | BIT(0));
		val = PAD257_CNTRL; /*rgmii1_txd[3]*/
		PAD257_CNTRL = (volatile unsigned int) (BIT(0));
		val = PAD258_CNTRL; /*rgmii1_rxd[2]*/
		PAD258_CNTRL = (volatile unsigned int) (BIT(18) | BIT(0));
#endif		
	}
}
#endif /* CONFIG_DRIVER_TI_CPSW */

void set_uart_mux_conf(void)
{
	/* Set UART pins */
	enable_uart0_pin_mux();
}
void set_muxconf_regs(void)
{
	u32 i, add, val;

	u32 pad_conf[] = {
		/* 1-4 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -8 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -12 */	BIT(0), BIT(0), BIT(0), BIT(0),
				/* 14 usb1_drvvbus, fn8, pulldn enable */
		/* -16 */	BIT(0), BIT(7), BIT(0), BIT(0),
		/* -20 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -24 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -28 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -32 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -36 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -40 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -44 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -48 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -52 */	BIT(0), BIT(0), BIT(7), BIT(7),/*54-AP_LED , 55-LED_RED by chlee*/
		/* -56 */	BIT(7), BIT(7), BIT(7), BIT(0),/*56-LED_YG,57-REC_LED,58-POWER_LED by chlee*/
		/* -60 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -64 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -68 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -72 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -76 */	BIT(6), BIT(6), BIT(6), BIT(0), /* 74 - mmc1_pow, 75 - mmc1_sdwp */ /* 73  hbchen sd2_sdcd */
		/* -80 */	BIT(0), BIT(0), BIT(0), BIT(1), /* 80 - mmc1_sdcd */
		/* -84 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -88 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -92 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -96 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -100 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -104 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -108 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -112 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -116 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -120 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -124 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -128 */	BIT(0), BIT(1), BIT(0), BIT(0), /* hbchen 126 - SD2_CMD */
		/* -132 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -136 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -140 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -144 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -148 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -152 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -156 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -160 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -164 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -168 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -172 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -176 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -180 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -184 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -188 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -192 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -196 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -200 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -204 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -208 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -212 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -216 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -220 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -224 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -228 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -232 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -236 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -240 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -244 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -248 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -252 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -256 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -260 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -264 */	BIT(0), BIT(0), BIT(0), BIT(0),
		/* -268 */	BIT(0), BIT(0), BIT(0), BIT(0),
				/* 270 usb0_drvvbus, fn1, pulldn enable */
		/* -271 */	BIT(0), BIT(0), BIT(0),

	};

	for (i = 0; i<N_PINS; i++)
	{
		add = PIN_CTRL_BASE + (i*4);
		val = __raw_readl(add);
		val |= pad_conf[i];
		__raw_writel(val, add);
	}
}


void set_mux_conf_regs(void)
{
	/* Set MMC pins */
//	enable_mmc1_pin_mux();

	/* Set Ethernet pins */
//	enable_enet_pin_mux();
	set_muxconf_regs();

	cpsw_pad_config();

		/* MMC/SD pull-down enable */
	__raw_writel(0x000C0040, 0x48140928);
	__raw_writel(0x000C0080, 0x48140924);//SD POWER
	
	__raw_writel(0x000c0080, 0x481408d4); // 54 GP0[24]
	__raw_writel(0x000c0080, 0x481408d8); // 55 GP0[25]
	__raw_writel(0x000c0080, 0x481408dc); // 56 GP0[26]
	__raw_writel(0x000c0080, 0x481408e0); // 57 GP0[27]
	__raw_writel(0x000c0080, 0x481408e4); // 58 GP0[28]
	
	led_init();
	gpio_direction_output(2+32,0);//set SD PWR to low

	__raw_writel(0x1, RMII_REFCLK_SRC); // orig for Corsica1&2
//	__raw_writel(0x3, RMII_REFCLK_SRC); // for Corsica 1 (A101)

	__raw_writel(0x106,GMII_SEL);//for RMII/RGMII hbchen
}

void sdram_init(void)
{
#if 0
	config_dmm(&evm_lisa_map_regs);

	config_ddr(0, NULL, &evm_ddr2_data, &evm_ddr2_cctrl_data,
		   &evm_ddr2_emif0_regs, 0);
	config_ddr(0, NULL, &evm_ddr2_data, &evm_ddr2_cctrl_data,
		   &evm_ddr2_emif1_regs, 1);
#else
	config_ti814x_ddr();

#endif
}
#endif



/*
 * Basic board specific setup.  Pinmux has been handled already.
 */
int board_init(void)
{
	u32 regVal;


	led_init();
	gpio_direction_output(2+32,0);//set SD PWR to low

	gd->bd->bi_boot_params = CONFIG_SYS_SDRAM_BASE + 0x100;
	__raw_writel(0x4, RMII_REFCLK_SRC); // orig
	__raw_writel(0x106,GMII_SEL);//for RMII/RGMII hbchen
	return 0;
}

#if !defined(CONFIG_SPL_BUILD) && defined(CONFIG_GENERIC_MMC)
int board_mmc_init(bd_t *bis)
{
	omap_mmc_init(0, 0, 0, -1, -1);

	omap_mmc_init(1, 0, 0, -1, -1);

	return 0;
}
#endif

#ifdef CONFIG_DRIVER_TI_CPSW
static void cpsw_control(int enabled)
{
	/* VTP can be added here */

	return;
}
static struct cpsw_slave_data cpsw_slaves[] = {
	{
		.slave_reg_ofs  = 0x50,
		.sliver_reg_ofs = 0x700,
		.phy_addr         = 2,
	},
	{
		.slave_reg_ofs  = 0x90,
		.sliver_reg_ofs = 0x740,
		.phy_addr         = 0x14,
	},
};


static struct cpsw_platform_data cpsw_data = {
	.mdio_base		= CPSW_MDIO_BASE,
	.cpsw_base		= CPSW_BASE,
	.mdio_div		= 0xff,
	.channels		= 8,
	.cpdma_reg_ofs		= 0x100,
	.slaves			= 1,
	.slave_data		= cpsw_slaves,
	.ale_reg_ofs		= 0x600,
	.ale_entries		= 1024,
	.host_port_reg_ofs	= 0x28,
	.hw_stats_reg_ofs	= 0x400,
	.bd_ram_ofs		= 0x2000,
	.mac_control		= (1 << 5),
	.control		= cpsw_control,
	.host_port_num		= 0,
	.version		= CPSW_CTRL_VERSION_1,
};
#endif

int board_eth_init(bd_t *bis)
{
	uint8_t mac_addr[6];
	uint32_t mac_hi, mac_lo;

	if (!eth_getenv_enetaddr("ethaddr", mac_addr)) {
		printf("<ethaddr> not set. Reading from E-fuse\n");
		/* try reading mac address from efuse */
		mac_lo = readl(&cdev->macid0l);
		mac_hi = readl(&cdev->macid0h);
		mac_addr[0] = mac_hi & 0xFF;
		mac_addr[1] = (mac_hi & 0xFF00) >> 8;
		mac_addr[2] = (mac_hi & 0xFF0000) >> 16;
		mac_addr[3] = (mac_hi & 0xFF000000) >> 24;
		mac_addr[4] = mac_lo & 0xFF;
		mac_addr[5] = (mac_lo & 0xFF00) >> 8;

		if (is_valid_ether_addr(mac_addr))
			eth_setenv_enetaddr("ethaddr", mac_addr);
		else
			printf("Unable to read MAC address. Set <ethaddr>\n");
	}

	return cpsw_register(&cpsw_data);
}
#ifdef CONFIG_SPL_OS_BOOT
int spl_start_uboot(void)
{
	/* break into full u-boot on 'c' */
	if (serial_tstc() && serial_getc() == 'c')
		return 1;

#ifdef CONFIG_SPL_ENV_SUPPORT
#if 0
	env_init();
	env_relocate_spec();
	if (getenv_yesno("boot_os") != 1)
		return 1;
#endif
#endif

	return 0;
}
#endif

